@@ -3,9 +3,9 @@
  *
  * TOMOYO Linux's utilities.
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
- * Version: 1.6.8   2009/05/28
+ * Version: 1.7.1+   2010/01/10
  *
  */
 #include "ccstools.h"
@@ -30,10 +30,6 @@ void editpolicy_color_init(void)
 		  COLOR_GREEN,      "DOMAIN_HEAD" },
 		{ DOMAIN_CURSOR,    COLOR_BLACK,
 		  COLOR_GREEN,      "DOMAIN_CURSOR" },
-		{ SYSTEM_HEAD,      COLOR_WHITE,
-		  COLOR_BLUE,       "SYSTEM_HEAD" },
-		{ SYSTEM_CURSOR,    COLOR_WHITE,
-		  COLOR_BLUE,       "SYSTEM_CURSOR" },
 		{ EXCEPTION_HEAD,   COLOR_BLACK,
 		  COLOR_CYAN,       "EXCEPTION_HEAD" },
 		{ EXCEPTION_CURSOR, COLOR_BLACK,
@@ -62,22 +58,25 @@ void editpolicy_color_init(void)
 	if (!fp)
 		goto use_default;
 	get();
-	while (freadline(fp)) {
+	while (true) {
+		char *line = freadline(fp);
 		char *cp;
-		if (!str_starts(shared_buffer, "editpolicy.line_color "))
+		if (!line)
+			break;
+		if (!str_starts(line, "editpolicy.line_color "))
 			continue;
-		cp = strchr(shared_buffer, '=');
+		cp = strchr(line, '=');
 		if (!cp)
 			continue;
 		*cp++ = '\0';
-		normalize_line(shared_buffer);
+		normalize_line(line);
 		normalize_line(cp);
-		if (!*shared_buffer || !*cp)
+		if (!*line || !*cp)
 			continue;
 		for (i = 0; color_env[i].name; i++) {
 			short int fore;
 			short int back;
-			if (strcmp(shared_buffer, color_env[i].name))
+			if (strcmp(line, color_env[i].name))
 				continue;
 			if (strlen(cp) != 2)
 				break;
@@ -141,8 +140,6 @@ int editpolicy_color_head(const int screen)
 	switch (screen) {
 	case SCREEN_DOMAIN_LIST:
 		return DOMAIN_HEAD;
-	case SCREEN_SYSTEM_LIST:
-		return SYSTEM_HEAD;
 	case SCREEN_EXCEPTION_LIST:
 		return EXCEPTION_HEAD;
 	case SCREEN_PROFILE_LIST:
@@ -161,8 +158,6 @@ int editpolicy_color_cursor(const int screen)
 	switch (screen) {
 	case SCREEN_DOMAIN_LIST:
 		return DOMAIN_CURSOR;
-	case SCREEN_SYSTEM_LIST:
-		return SYSTEM_CURSOR;
 	case SCREEN_EXCEPTION_LIST:
 		return EXCEPTION_CURSOR;
 	case SCREEN_PROFILE_LIST:
@@ -179,9 +174,9 @@ int editpolicy_color_cursor(const int screen)
 void editpolicy_line_draw(const int screen)
 {
 	static int before_current[MAXSCREEN] = { -1, -1, -1, -1,
-						 -1, -1, -1, -1 };
+						 -1, -1, -1 };
 	static int before_y[MAXSCREEN]       = { -1, -1, -1, -1,
-						 -1, -1, -1, -1 };
+						 -1, -1, -1 };
 	int current = editpolicy_get_current();
 	int y;
 	int x;
